@@ -15,9 +15,10 @@ A.setSizes([n**3, n**3])
 A.setType('python')
 shell = Del2Mat(n) # shell context
 A.setPythonContext(shell)
+A.setUp()
 
 # setup linear system vectors
-x, b = A.getVecs()
+x, b = A.createVecs()
 x.set(0.0)
 b.set(1.0)
 
@@ -37,14 +38,17 @@ ksp.solve(b, x)
 # account for grid spacing
 x.scale(h**2)
 
-try:
-    from matplotlib import pylab
-except ImportError:
-    raise SystemExit("matplotlib not available")
-from numpy import mgrid
-X, Y =  mgrid[0:1:1j*n,0:1:1j*n]
-Z = x[...].reshape(n,n,n)[:,:,n/2-2]
-pylab.contourf(X, Y, Z)
-pylab.axis('equal')
-pylab.colorbar()
-pylab.show()
+OptDB = PETSc.Options()
+if OptDB.getBool('plot_mpl', False):
+    try:
+        from matplotlib import pylab
+    except ImportError:
+        PETSc.Sys.Print("matplotlib not available")
+    else:
+        from numpy import mgrid
+        X, Y =  mgrid[0:1:1j*n,0:1:1j*n]
+        Z = x[...].reshape(n,n,n)[:,:,n/2-2]
+        pylab.contourf(X, Y, Z)
+        pylab.axis('equal')
+        pylab.colorbar()
+        pylab.show()
