@@ -15,24 +15,25 @@ solution of scientific applications modeled by partial differential
 equations. It employs the MPI_ standard for all message-passing
 communication.
 
-.. _PETSc: http://www-unix.mcs.anl.gov/petsc
+.. _PETSc: http://www.mcs.anl.gov/petsc
 .. _MPI:   http://www.mpi-forum.org
 
 """
 
 __author__    = 'Lisandro Dalcin'
-__version__   = '1.1'
-__credits__   = "PETSc Team <petsc-maint@mcs.anl.gov>"
+__version__   = '3.7.0'
+__credits__   = 'PETSc Team <petsc-maint@mcs.anl.gov>'
 
 # --------------------------------------------------------------------
 
-def init(args=None, arch=None):
+def init(args=None, arch=None, comm=None):
     """
     Initialize PETSc.
 
     :Parameters:
       - `args`: command-line arguments, usually the 'sys.argv' list.
       - `arch`: specific configuration to use.
+      - `comm`: MPI commmunicator
 
     .. note:: This function should be called only once, typically at
        the very beginning of the bootstrap script of an application.
@@ -40,7 +41,7 @@ def init(args=None, arch=None):
     import petsc4py.lib
     PETSc = petsc4py.lib.ImportPETSc(arch)
     args  = petsc4py.lib.getInitArgs(args)
-    PETSc._initialize(args)
+    PETSc._initialize(args, comm)
 
 # --------------------------------------------------------------------
 
@@ -60,9 +61,3 @@ def get_include():
     return join(dirname(__file__), 'include')
 
 # --------------------------------------------------------------------
-
-if __name__ == '__main__':
-    import petsc4py.help
-    petsc4py.help.help()
-
-# --------------------------------------------------------------------
