@@ -5,17 +5,7 @@
 #define PETSC4PY_H
 
 #include <Python.h>
-
 #include <petsc.h>
-#include <petscis.h>
-#include <petscvec.h>
-#include <petscmat.h>
-#include <petscksp.h>
-#include <petscsnes.h>
-#include <petscts.h>
-#include <petscao.h>
-#include <petscda.h>
-
 #include "petsc4py.PETSc_api.h"
 
 static int import_petsc4py(void) {
