@@ -8,7 +8,7 @@ def mkgraph(comm, m, n):
     end   = start + m*n
     idt = PETSc.IntType
     rows = []
-    for I in xrange(start, end) :
+    for I in range(start, end) :
         rows.append([])
         adj = rows[-1]
         i = I//n; j = I - i*n
@@ -36,18 +36,25 @@ class BaseTestMatAnyAIJ(object):
     def setUp(self):
         COMM   = self.COMM
         GM, GN = self.GRID
-        BS     = self.BSIZE or 1
+        BS     = self.BSIZE
         #
-        sdt = dtype=PETSc.ScalarType
+        try:
+            rbs, cbs = BS
+            rbs = rbs or 1
+            cbs = cbs or 1
+        except (TypeError, ValueError):
+            rbs = cbs = BS or 1
+        sdt = dtype = PETSc.ScalarType
         self.rows, self.xadj, self.adjy = mkgraph(COMM, GM, GN)
-        self.vals = N.array(range(1, 1 + len(self.adjy)* BS**2), dtype=sdt)
-        self.vals.shape = (-1, BS, BS)
+        self.vals = N.array(range(1, 1 + len(self.adjy)* rbs*cbs), dtype=sdt)
+        self.vals.shape = (-1, rbs, cbs)
         #
-        self.A = A = PETSc.Mat().create(comm=COMM)
-        bs = BS; m, n = GM, GN; cs = COMM.getSize()
-        rowsz = colsz = (m*n*bs, None)
-        A.setSizes([rowsz, colsz], bs)
+        m, n = GM, GN
+        rowsz = (m*n*rbs, None)
+        colsz = (m*n*cbs, None)
+        A = self.A = PETSc.Mat().create(comm=COMM)
         A.setType(self.TYPE)
+        A.setSizes([rowsz, colsz], BS)
 
     def tearDown(self):
         self.A.destroy()
@@ -55,7 +62,7 @@ class BaseTestMatAnyAIJ(object):
 
     def testSetPreallocNNZ(self):
         nnz = [5, 2]
-        self.A.setPreallocationNNZ(nnz, self.BSIZE)
+        self.A.setPreallocationNNZ(nnz)
         self._chk_bs(self.A, self.BSIZE)
         opt = PETSc.Mat.Option.NEW_NONZERO_ALLOCATION_ERR
         self.A.setOption(opt, True)
@@ -72,7 +79,7 @@ class BaseTestMatAnyAIJ(object):
         _, ai, _, _ =self._get_aijv()
         d_nnz = N.diff(ai)
         nnz = [d_nnz, 3]
-        self.A.setPreallocationNNZ(nnz, self.BSIZE)
+        self.A.setPreallocationNNZ(nnz)
         self._chk_bs(self.A, self.BSIZE)
         opt = PETSc.Mat.Option.NEW_NONZERO_ALLOCATION_ERR
         self.A.setOption(opt, True)
@@ -86,9 +93,9 @@ class BaseTestMatAnyAIJ(object):
         self._chk_aij(self.A, ai, aj)
 
     def testSetPreallocCSR(self):
-        _, ai, aj, _ =self._get_aijv()
+        _, ai, aj, _ = self._get_aijv()
         csr = [ai, aj]
-        self.A.setPreallocationCSR(csr, self.BSIZE)
+        self.A.setPreallocationCSR(csr)
         self._chk_bs(self.A, self.BSIZE)
         self._chk_aij(self.A, ai, aj)
         opt = PETSc.Mat.Option.NEW_NONZERO_LOCATION_ERR
@@ -103,7 +110,7 @@ class BaseTestMatAnyAIJ(object):
     def testSetPreallocCSR_2(self):
         _, ai, aj, av =self._get_aijv()
         csr = [ai, aj, av]
-        self.A.setPreallocationCSR(csr, self.BSIZE)
+        self.A.setPreallocationCSR(csr)
         self._chk_bs(self.A, self.BSIZE)
         self._chk_aij(self.A, ai, aj)
         opt = PETSc.Mat.Option.NEW_NONZERO_LOCATION_ERR
@@ -141,6 +148,29 @@ class BaseTestMatAnyAIJ(object):
         self.A.assemble()
         self._chk_aij(self.A, ai, aj)
 
+    def testGetValuesCSR(self):
+        self._preallocate()
+        self._set_values_ijv()
+        A = self.A
+        A.assemble()
+        if 'sbaij' in A.getType():
+            opt = PETSc.Mat.Option.GETROW_UPPERTRIANGULAR
+            self.A.setOption(opt, True)
+        ai, aj, av = A.getValuesCSR()
+        rstart, rend = A.getOwnershipRange()
+        for row in range(rstart, rend):
+            cols, vals = A.getRow(row)
+            i = row - rstart
+            self.assertTrue(N.allclose(aj[ai[i]:ai[i+1]], cols))
+            self.assertTrue(N.allclose(av[ai[i]:ai[i+1]], vals))
+
+    #def testConvertToAIJ(self):
+    #    self._preallocate()
+    #    self._set_values_ijv()
+    #    A = self.A
+    #    A.assemble()
+    #    A.convert('aij')
+
     def testGetDiagonalBlock(self):
         self._preallocate()
         self._set_values_ijv()
@@ -149,6 +179,31 @@ class BaseTestMatAnyAIJ(object):
         self.assertEqual(self.A.getLocalSize(), B.getSize())
         B.destroy()
 
+    def testInvertBlockDiagonal(self):
+        try:
+            _ = len(self.BSIZE)
+            return
+        except (TypeError, ValueError):
+            pass
+        self._preallocate()
+        rbs, cbs = self.A.getBlockSizes()
+        if rbs != cbs: return
+        self._set_values_ijv()
+        self.A.assemble()
+        self.A.shift(1000) # Make nonsingular
+        ibdiag = self.A.invertBlockDiagonal()
+        bs = self.A.getBlockSize()
+        m, _ = self.A.getLocalSize()
+        self.assertEqual(ibdiag.shape, (m//bs, bs, bs))
+        tmp = N.empty((m//bs, bs, bs), dtype=PETSc.ScalarType)
+        rstart, rend = self.A.getOwnershipRange()
+        s, e = rstart//bs, rend//bs
+        for i in range(s, e):
+            rows = cols = N.arange(i*bs,(i+1)*bs, dtype=PETSc.IntType)
+            vals = self.A.getValues(rows,cols)
+            tmp[i-s,:,:] = N.linalg.inv(vals)
+        self.assertTrue(N.allclose(ibdiag, tmp))
+
     def testGetSubMatrix(self):
         if 'baij' in self.A.getType(): return # XXX
         self._preallocate()
@@ -158,8 +213,8 @@ class BaseTestMatAnyAIJ(object):
         rank = self.A.getComm().getRank()
         rs, re = self.A.getOwnershipRange()
         cs, ce = self.A.getOwnershipRangeColumn()
-        rows = N.arange(rs, re, dtype=PETSc.IntType)
-        cols = N.arange(cs, ce, dtype=PETSc.IntType)
+        rows = N.array(range(rs, re), dtype=PETSc.IntType)
+        cols = N.array(range(cs, ce), dtype=PETSc.IntType)
         rows = PETSc.IS().createGeneral(rows, comm=self.A.getComm())
         cols = PETSc.IS().createGeneral(cols, comm=self.A.getComm())
         #
@@ -173,14 +228,63 @@ class BaseTestMatAnyAIJ(object):
         self.A.getSubMatrix(rows, cols, S)
         S.destroy()
 
+    def testGetSubMatrices(self):
+        if 'baij' in self.A.getType(): return # XXX
+        self._preallocate()
+        self._set_values_ijv()
+        self.A.assemble()
+        #
+        rs, re = self.A.getOwnershipRange()
+        cs, ce = self.A.getOwnershipRangeColumn()
+        rows = N.array(range(rs, re), dtype=PETSc.IntType)
+        cols = N.array(range(cs, ce), dtype=PETSc.IntType)
+        rows = PETSc.IS().createGeneral(rows, comm=self.A.getComm())
+        cols = PETSc.IS().createGeneral(cols, comm=self.A.getComm())
+        #
+        (S,) = self.A.getSubMatrices(rows, cols)
+        S.zeroEntries()
+        self.A.getSubMatrices(rows, cols, submats=[S])
+        S.destroy()
+        #
+        (S1,) = self.A.getSubMatrices([rows], [cols])
+        (S2,) = self.A.getSubMatrices([rows], [cols])
+        self.assertTrue(S1.equal(S2))
+        S2.zeroEntries()
+        self.A.getSubMatrices([rows], [cols], [S2])
+        self.assertTrue(S1.equal(S2))
+        S1.destroy()
+        S2.destroy()
+        #
+        if 'seq' not in self.A.getType(): return # XXX
+        S1, S2 = self.A.getSubMatrices([rows, rows], [cols, cols])
+        self.assertTrue(S1.equal(S2))
+        S1.zeroEntries()
+        S2.zeroEntries()
+        self.A.getSubMatrices([rows, rows], [cols, cols], [S1, S2])
+        self.assertTrue(S1.equal(S2))
+        S1.destroy()
+        S2.destroy()
+
+    def testGetRedundantMatrix(self):
+        if 'aijcrl' in self.A.getType(): return # duplicate not supported
+        self._preallocate()
+        self._set_values_ijv()
+        self.A.assemble()
+        #Test the most simple case
+        sizecommA = self.A.getComm().getSize()
+        Ared = self.A.getRedundantMatrix(sizecommA)
+        sizecommAred = Ared.getComm().getSize()
+        self.assertEqual(1, sizecommAred)
+        Ared.destroy()
+
     def testCreateTranspose(self):
         self._preallocate()
         self._set_values_ijv()
         self.A.assemble()
         A = self.A
         AT = PETSc.Mat().createTranspose(A)
-        x, y = A.getVecs()
-        xt, yt = AT.getVecs()
+        x, y = A.createVecs()
+        xt, yt = AT.createVecs()
         #
         y.setRandom()
         A.multTranspose(y, x)
@@ -198,7 +302,7 @@ class BaseTestMatAnyAIJ(object):
         return (self.rows, self.xadj, self.adjy, self.vals,)
 
     def _preallocate(self):
-        self.A.setPreallocationNNZ([5, 2], self.BSIZE)
+        self.A.setPreallocationNNZ([5, 2])
 
     def _set_values(self):
         import sys
@@ -228,21 +332,23 @@ class BaseTestMatAnyAIJ(object):
     def _chk_bs(self, A, bs):
         self.assertEqual(A.getBlockSize(), bs or 1)
 
-    def _chk_aij(self, A, i, j):
-        ai, aj = A.getRowIJ(compressed=bool(self.BSIZE))
-        if None not in (ai, aj):
-            self.assertTrue(N.all(i==ai))
-            self.assertTrue(N.all(j==aj))
-        ai, aj = A.getColumnIJ(compressed=bool(self.BSIZE))
-        if None not in (ai, aj):
-            self.assertTrue(N.all(i==ai))
-            self.assertTrue(N.all(j==aj))
-        return # XXX review
-        version, patch = PETSc.Sys.getVersion(patch=True)
-        #if version >= (2,3,3): #and patch >= 13:
-        ai, aj = A.getRowIJ(compressed=False)
-        print ai, aj
+    def _chk_bsizes(self, A, bsizes):
+        try:
+            rbs, cbs = bsizes
+        except (TypeError, ValueError):
+            rbs = cbs = bsizes
+        self.assertEqual(A.getBlockSizes(), (rbs, cbs))
 
+    def _chk_aij(self, A, i, j):
+        compressed = bool(self.BSIZE)
+        ai, aj = A.getRowIJ(compressed=compressed)
+        if ai is not None and aj is not None:
+            self.assertTrue(N.all(i==ai))
+            self.assertTrue(N.all(j==aj))
+        ai, aj = A.getColumnIJ(compressed=compressed)
+        if ai is not None and aj is not None:
+            self.assertTrue(N.all(i==ai))
+            self.assertTrue(N.all(j==aj))
 
 # -- AIJ ---------------------
 
@@ -368,6 +474,108 @@ class TestMatMPIBAIJ_G45_B5(TestMatMPIBAIJ_G45):
 class TestMatMPIBAIJ_G89_B5(TestMatMPIBAIJ_G89):
     BSIZE = 5
 
+# -- SymmBlock AIJ ---------------
+
+class BaseTestMatSBAIJ(BaseTestMatAnyAIJ, unittest.TestCase):
+    COMM  = PETSc.COMM_WORLD
+    TYPE  = PETSc.Mat.Type.SBAIJ
+    GRID  = 0, 0
+    BSIZE = 1
+    def testInvertBlockDiagonal(self): pass
+    def _chk_aij(self, A, i, j):
+        ai, aj = A.getRowIJ(compressed=True)
+        if ai is not None and aj is not None:
+            if 0: # XXX Implement
+                self.assertTrue(N.all(i==ai))
+                self.assertTrue(N.all(j==aj))
+        ai, aj = A.getColumnIJ(compressed=True)
+        if ai is not None and aj is not None:
+            if 0: # XXX Implement
+                self.assertTrue(N.all(i==ai))
+                self.assertTrue(N.all(j==aj))
+
+# -- Seq SymmBlock AIJ --
+
+class TestMatSeqSBAIJ(BaseTestMatSBAIJ):
+    COMM = PETSc.COMM_SELF
+    TYPE = PETSc.Mat.Type.SEQSBAIJ
+# bs = 1
+class TestMatSeqSBAIJ_G23(TestMatSeqSBAIJ):
+    GRID  = 2, 3
+class TestMatSeqSBAIJ_G45(TestMatSeqSBAIJ):
+    GRID  = 4, 5
+class TestMatSeqSBAIJ_G89(TestMatSeqSBAIJ):
+    GRID  = 8, 9
+# bs = 2
+class TestMatSeqSBAIJ_G23_B2(TestMatSeqSBAIJ_G23):
+    BSIZE = 2
+class TestMatSeqSBAIJ_G45_B2(TestMatSeqSBAIJ_G45):
+    BSIZE = 2
+class TestMatSeqSBAIJ_G89_B2(TestMatSeqSBAIJ_G89):
+    BSIZE = 2
+# bs = 3
+class TestMatSeqSBAIJ_G23_B3(TestMatSeqSBAIJ_G23):
+    BSIZE = 3
+class TestMatSeqSBAIJ_G45_B3(TestMatSeqSBAIJ_G45):
+    BSIZE = 3
+class TestMatSeqSBAIJ_G89_B3(TestMatSeqSBAIJ_G89):
+    BSIZE = 3
+# bs = 4
+class TestMatSeqSBAIJ_G23_B4(TestMatSeqSBAIJ_G23):
+    BSIZE = 4
+class TestMatSeqSBAIJ_G45_B4(TestMatSeqSBAIJ_G45):
+    BSIZE = 4
+class TestMatSeqSBAIJ_G89_B4(TestMatSeqSBAIJ_G89):
+    BSIZE = 4
+# bs = 5
+class TestMatSeqSBAIJ_G23_B5(TestMatSeqSBAIJ_G23):
+    BSIZE = 5
+class TestMatSeqSBAIJ_G45_B5(TestMatSeqSBAIJ_G45):
+    BSIZE = 5
+class TestMatSeqSBAIJ_G89_B5(TestMatSeqSBAIJ_G89):
+    BSIZE = 5
+
+
+# -- MPI SymmBlock AIJ --
+
+class TestMatMPISBAIJ(BaseTestMatSBAIJ):
+    COMM = PETSc.COMM_WORLD
+    TYPE = PETSc.Mat.Type.MPISBAIJ
+# bs = 1
+class TestMatMPISBAIJ_G23(TestMatMPISBAIJ):
+    GRID  = 2, 3
+class TestMatMPISBAIJ_G45(TestMatMPISBAIJ):
+    GRID  = 4, 5
+class TestMatMPISBAIJ_G89(TestMatMPISBAIJ):
+    GRID  = 8, 9
+# bs = 2
+class TestMatMPISBAIJ_G23_B2(TestMatMPISBAIJ_G23):
+    BSIZE = 2
+class TestMatMPISBAIJ_G45_B2(TestMatMPISBAIJ_G45):
+    BSIZE = 2
+class TestMatMPISBAIJ_G89_B2(TestMatMPISBAIJ_G89):
+    BSIZE = 2
+# bs = 3
+class TestMatMPISBAIJ_G23_B3(TestMatMPISBAIJ_G23):
+    BSIZE = 3
+class TestMatMPISBAIJ_G45_B3(TestMatMPISBAIJ_G45):
+    BSIZE = 3
+class TestMatMPISBAIJ_G89_B3(TestMatMPISBAIJ_G89):
+    BSIZE = 3
+# bs = 4
+class TestMatMPISBAIJ_G23_B4(TestMatMPISBAIJ_G23):
+    BSIZE = 4
+class TestMatMPISBAIJ_G45_B4(TestMatMPISBAIJ_G45):
+    BSIZE = 4
+class TestMatMPISBAIJ_G89_B4(TestMatMPISBAIJ_G89):
+    BSIZE = 4
+# bs = 5
+class TestMatMPISBAIJ_G23_B5(TestMatMPISBAIJ_G23):
+    BSIZE = 5
+class TestMatMPISBAIJ_G45_B5(TestMatMPISBAIJ_G45):
+    BSIZE = 5
+class TestMatMPISBAIJ_G89_B5(TestMatMPISBAIJ_G89):
+    BSIZE = 5
 
 # -- AIJ + Block ---------------
 
@@ -407,17 +615,16 @@ class BaseTestMatAIJ_B(BaseTestMatAnyAIJ, unittest.TestCase):
         self._chk_aij(self.A, ai, aj)
     def _preallocate(self):
         self.A.setPreallocationNNZ([5*self.BSIZE, 3*self.BSIZE])
-        self.A.setBlockSize(self.BSIZE)
         self._chk_bs(self.A, self.BSIZE)
     def _chk_aij(self, A, i, j):
         bs = self.BSIZE or 1
         ai, aj = A.getRowIJ()
-        if None not in (ai, aj):  ## XXX map and check !!
+        if ai is not None and aj is not None:  ## XXX map and check !!
             #self.assertTrue(N.all(i==ai))
             #self.assertTrue(N.all(j==aj))
             pass
         ai, aj = A.getColumnIJ(compressed=bool(self.BSIZE))
-        if None not in (ai, aj): ## XXX map and check !!
+        if ai is not None and aj is not None: ## XXX map and check !!
             #self.assertTrue(N.all(i==ai))
             #self.assertTrue(N.all(j==aj))
             pass
@@ -505,131 +712,180 @@ class TestMatMPIAIJ_B_G45_B5(TestMatMPIAIJ_B_G45):
 class TestMatMPIAIJ_B_G89_B5(TestMatMPIAIJ_B_G89):
     BSIZE = 5
 
+# -- Non-square blocks --
+class BaseTestMatAIJ_B(BaseTestMatAnyAIJ, unittest.TestCase):
+    COMM  = PETSc.COMM_WORLD
+    TYPE  = PETSc.Mat.Type.AIJ
+    GRID  = 0, 0
+    BSIZE = 4, 2
+
+    def _preallocate(self):
+        try:
+            rbs, cbs = self.BSIZE
+        except (TypeError, ValueError):
+            rbs = cbs = self.BSIZE
+        self.A.setPreallocationNNZ([5*rbs, 3*cbs])
+        self._chk_bsizes(self.A, self.BSIZE)
+    def testSetPreallocNNZ(self):pass
+    def testSetPreallocNNZ_2(self):pass
+    def testSetPreallocCSR(self):pass
+    def testSetPreallocCSR_2(self):pass
+    def testSetValues(self):
+        self._preallocate()
+        opt = PETSc.Mat.Option.NEW_NONZERO_ALLOCATION_ERR
+        self.A.setOption(opt, True)
+        ai, aj, av = self._set_values()
+        self.A.assemble()
+        self._chk_aij(self.A, ai, aj)
+        opt = PETSc.Mat.Option.NEW_NONZERO_LOCATION_ERR
+        self.A.setOption(opt, True)
+        ai, aj, av = self._set_values()
+        self.A.assemble()
+        self._chk_aij(self.A, ai, aj)
+    def testSetValuesIJV(self):
+        self._preallocate()
+        opt = PETSc.Mat.Option.NEW_NONZERO_ALLOCATION_ERR
+        self.A.setOption(opt, True)
+        ai, aj, av = self._set_values_ijv()
+        self.A.assemble()
+        self._chk_aij(self.A, ai, aj)
+        opt = PETSc.Mat.Option.NEW_NONZERO_LOCATION_ERR
+        self.A.setOption(opt, True)
+        ai, aj, av = self._set_values_ijv()
+        self.A.assemble()
+        self._chk_aij(self.A, ai, aj)
+    def _chk_aij(self, A, i, j):
+        bs = self.BSIZE or 1
+        ai, aj = A.getRowIJ()
+        if ai is not None and aj is not None:  ## XXX map and check !!
+            #self.assertTrue(N.all(i==ai))
+            #self.assertTrue(N.all(j==aj))
+            pass
+        ai, aj = A.getColumnIJ()
+        if ai is not None and aj is not None: ## XXX map and check !!
+            #self.assertTrue(N.all(i==ai))
+            #self.assertTrue(N.all(j==aj))
+            pass
+
+# -- AIJCRL ---------------------
+
+class BaseTestMatAIJCRL(BaseTestMatAIJ, unittest.TestCase):
+    TYPE  = PETSc.Mat.Type.AIJCRL
+
+# -- Seq AIJCRL --
+
+class TestMatSeqAIJCRL(BaseTestMatAIJCRL):
+    COMM = PETSc.COMM_SELF
+    TYPE = PETSc.Mat.Type.SEQAIJCRL
+class TestMatSeqAIJCRL_G23(TestMatSeqAIJCRL):
+    GRID  = 2, 3
+class TestMatSeqAIJCRL_G45(TestMatSeqAIJCRL):
+    GRID  = 4, 5
+class TestMatSeqAIJCRL_G89(TestMatSeqAIJCRL):
+    GRID  = 8, 9
+
+# -- MPI AIJCRL --
+
+class TestMatMPIAIJCRL(BaseTestMatAIJCRL):
+    COMM = PETSc.COMM_WORLD
+    TYPE = PETSc.Mat.Type.MPIAIJCRL
+class TestMatMPIAIJCRL_G23(TestMatMPIAIJCRL):
+    GRID  = 2, 3
+class TestMatMPIAIJCRL_G45(TestMatMPIAIJCRL):
+    GRID  = 4, 5
+class TestMatMPIAIJCRL_G89(TestMatMPIAIJCRL):
+    GRID  = 8, 9
+
+# -- AIJCRL + Block -------------
+
+class BaseTestMatAIJCRL_B(BaseTestMatAIJ_B, unittest.TestCase):
+    TYPE  = PETSc.Mat.Type.AIJCRL
+
+# -- Seq AIJCRL + Block --
+
+class TestMatSeqAIJCRL_B(BaseTestMatAIJCRL_B):
+    COMM = PETSc.COMM_SELF
+    TYPE = PETSc.Mat.Type.SEQAIJCRL
+# bs = 1
+class TestMatSeqAIJCRL_B_G23(TestMatSeqAIJCRL_B):
+    GRID  = 2, 3
+class TestMatSeqAIJCRL_B_G45(TestMatSeqAIJCRL_B):
+    GRID  = 4, 5
+class TestMatSeqAIJCRL_B_G89(TestMatSeqAIJCRL_B):
+    GRID  = 8, 9
+# bs = 2
+class TestMatSeqAIJCRL_B_G23_B2(TestMatSeqAIJCRL_B_G23):
+    BSIZE = 2
+class TestMatSeqAIJCRL_B_G45_B2(TestMatSeqAIJCRL_B_G45):
+    BSIZE = 2
+class TestMatSeqAIJCRL_B_G89_B2(TestMatSeqAIJCRL_B_G89):
+    BSIZE = 2
+# bs = 3
+class TestMatSeqAIJCRL_B_G23_B3(TestMatSeqAIJCRL_B_G23):
+    BSIZE = 3
+class TestMatSeqAIJCRL_B_G45_B3(TestMatSeqAIJCRL_B_G45):
+    BSIZE = 3
+class TestMatSeqAIJCRL_B_G89_B3(TestMatSeqAIJCRL_B_G89):
+    BSIZE = 3
+# bs = 4
+class TestMatSeqAIJCRL_B_G23_B4(TestMatSeqAIJCRL_B_G23):
+    BSIZE = 4
+class TestMatSeqAIJCRL_B_G45_B4(TestMatSeqAIJCRL_B_G45):
+    BSIZE = 4
+class TestMatSeqAIJCRL_B_G89_B4(TestMatSeqAIJCRL_B_G89):
+    BSIZE = 4
+# bs = 5
+class TestMatSeqAIJCRL_B_G23_B5(TestMatSeqAIJCRL_B_G23):
+    BSIZE = 5
+class TestMatSeqAIJCRL_B_G45_B5(TestMatSeqAIJCRL_B_G45):
+    BSIZE = 5
+class TestMatSeqAIJCRL_B_G89_B5(TestMatSeqAIJCRL_B_G89):
+    BSIZE = 5
+
+
+# -- MPI AIJCRL + Block --
+
+class TestMatMPIAIJCRL_B(BaseTestMatAIJCRL_B):
+    COMM = PETSc.COMM_WORLD
+    TYPE = PETSc.Mat.Type.MPIAIJCRL
+# bs = 1
+class TestMatMPIAIJCRL_B_G23(TestMatMPIAIJCRL_B):
+    GRID  = 2, 3
+class TestMatMPIAIJCRL_B_G45(TestMatMPIAIJCRL_B):
+    GRID  = 4, 5
+class TestMatMPIAIJCRL_B_G89(TestMatMPIAIJCRL_B):
+    GRID  = 8, 9
+# bs = 2
+class TestMatMPIAIJCRL_B_G23_B2(TestMatMPIAIJCRL_B_G23):
+    BSIZE = 2
+class TestMatMPIAIJCRL_B_G45_B2(TestMatMPIAIJCRL_B_G45):
+    BSIZE = 2
+class TestMatMPIAIJCRL_B_G89_B2(TestMatMPIAIJCRL_B_G89):
+    BSIZE = 2
+# bs = 3
+class TestMatMPIAIJCRL_B_G23_B3(TestMatMPIAIJCRL_B_G23):
+    BSIZE = 3
+class TestMatMPIAIJCRL_B_G45_B3(TestMatMPIAIJCRL_B_G45):
+    BSIZE = 3
+class TestMatMPIAIJCRL_B_G89_B3(TestMatMPIAIJCRL_B_G89):
+    BSIZE = 3
+# bs = 4
+class TestMatMPIAIJCRL_B_G23_B4(TestMatMPIAIJCRL_B_G23):
+    BSIZE = 4
+class TestMatMPIAIJCRL_B_G45_B4(TestMatMPIAIJCRL_B_G45):
+    BSIZE = 4
+class TestMatMPIAIJCRL_B_G89_B4(TestMatMPIAIJCRL_B_G89):
+    BSIZE = 4
+# bs = 5
+class TestMatMPIAIJCRL_B_G23_B5(TestMatMPIAIJCRL_B_G23):
+    BSIZE = 5
+class TestMatMPIAIJCRL_B_G45_B5(TestMatMPIAIJCRL_B_G45):
+    BSIZE = 5
+class TestMatMPIAIJCRL_B_G89_B5(TestMatMPIAIJCRL_B_G89):
+    BSIZE = 5
+
 # -----
 
-if PETSc.Sys.getVersion() < (3,0,0):
-    del BaseTestMatAnyAIJ.testCreateTranspose
-
-if PETSc.Sys.getVersion() >= (3,1,0):
-    # -- CRL ---------------------
-
-    class BaseTestMatCRL(BaseTestMatAIJ, unittest.TestCase):
-        TYPE  = PETSc.Mat.Type.CRL
-
-    # -- Seq CRL --
-
-    class TestMatSeqCRL(BaseTestMatCRL):
-        COMM = PETSc.COMM_SELF
-        TYPE = PETSc.Mat.Type.SEQCRL
-    class TestMatSeqCRL_G23(TestMatSeqCRL):
-        GRID  = 2, 3
-    class TestMatSeqCRL_G45(TestMatSeqCRL):
-        GRID  = 4, 5
-    class TestMatSeqCRL_G89(TestMatSeqCRL):
-        GRID  = 8, 9
-
-    # -- MPI CRL --
-
-    class TestMatMPICRL(BaseTestMatCRL):
-        COMM = PETSc.COMM_WORLD
-        TYPE = PETSc.Mat.Type.MPICRL
-    class TestMatMPICRL_G23(TestMatMPICRL):
-        GRID  = 2, 3
-    class TestMatMPICRL_G45(TestMatMPICRL):
-        GRID  = 4, 5
-    class TestMatMPICRL_G89(TestMatMPICRL):
-        GRID  = 8, 9
-
-    # -- CRL + Block -------------
-
-    class BaseTestMatCRL_B(BaseTestMatAIJ_B, unittest.TestCase):
-        TYPE  = PETSc.Mat.Type.AIJ
-
-    # -- Seq CRL + Block --
-
-    class TestMatSeqCRL_B(BaseTestMatCRL_B):
-        COMM = PETSc.COMM_SELF
-        TYPE = PETSc.Mat.Type.SEQCRL
-    # bs = 1
-    class TestMatSeqCRL_B_G23(TestMatSeqCRL_B):
-        GRID  = 2, 3
-    class TestMatSeqCRL_B_G45(TestMatSeqCRL_B):
-        GRID  = 4, 5
-    class TestMatSeqCRL_B_G89(TestMatSeqCRL_B):
-        GRID  = 8, 9
-    # bs = 2
-    class TestMatSeqCRL_B_G23_B2(TestMatSeqCRL_B_G23):
-        BSIZE = 2
-    class TestMatSeqCRL_B_G45_B2(TestMatSeqCRL_B_G45):
-        BSIZE = 2
-    class TestMatSeqCRL_B_G89_B2(TestMatSeqCRL_B_G89):
-        BSIZE = 2
-    # bs = 3
-    class TestMatSeqCRL_B_G23_B3(TestMatSeqCRL_B_G23):
-        BSIZE = 3
-    class TestMatSeqCRL_B_G45_B3(TestMatSeqCRL_B_G45):
-        BSIZE = 3
-    class TestMatSeqCRL_B_G89_B3(TestMatSeqCRL_B_G89):
-        BSIZE = 3
-    # bs = 4
-    class TestMatSeqCRL_B_G23_B4(TestMatSeqCRL_B_G23):
-        BSIZE = 4
-    class TestMatSeqCRL_B_G45_B4(TestMatSeqCRL_B_G45):
-        BSIZE = 4
-    class TestMatSeqCRL_B_G89_B4(TestMatSeqCRL_B_G89):
-        BSIZE = 4
-    # bs = 5
-    class TestMatSeqCRL_B_G23_B5(TestMatSeqCRL_B_G23):
-        BSIZE = 5
-    class TestMatSeqCRL_B_G45_B5(TestMatSeqCRL_B_G45):
-        BSIZE = 5
-    class TestMatSeqCRL_B_G89_B5(TestMatSeqCRL_B_G89):
-        BSIZE = 5
-
-
-    # -- MPI CRL + Block --
-
-    class TestMatMPICRL_B(BaseTestMatCRL_B):
-        COMM = PETSc.COMM_WORLD
-        TYPE = PETSc.Mat.Type.MPICRL
-    # bs = 1
-    class TestMatMPICRL_B_G23(TestMatMPICRL_B):
-        GRID  = 2, 3
-    class TestMatMPICRL_B_G45(TestMatMPICRL_B):
-        GRID  = 4, 5
-    class TestMatMPICRL_B_G89(TestMatMPICRL_B):
-        GRID  = 8, 9
-    # bs = 2
-    class TestMatMPICRL_B_G23_B2(TestMatMPICRL_B_G23):
-        BSIZE = 2
-    class TestMatMPICRL_B_G45_B2(TestMatMPICRL_B_G45):
-        BSIZE = 2
-    class TestMatMPICRL_B_G89_B2(TestMatMPICRL_B_G89):
-        BSIZE = 2
-    # bs = 3
-    class TestMatMPICRL_B_G23_B3(TestMatMPICRL_B_G23):
-        BSIZE = 3
-    class TestMatMPICRL_B_G45_B3(TestMatMPICRL_B_G45):
-        BSIZE = 3
-    class TestMatMPICRL_B_G89_B3(TestMatMPICRL_B_G89):
-        BSIZE = 3
-    # bs = 4
-    class TestMatMPICRL_B_G23_B4(TestMatMPICRL_B_G23):
-        BSIZE = 4
-    class TestMatMPICRL_B_G45_B4(TestMatMPICRL_B_G45):
-        BSIZE = 4
-    class TestMatMPICRL_B_G89_B4(TestMatMPICRL_B_G89):
-        BSIZE = 4
-    # bs = 5
-    class TestMatMPICRL_B_G23_B5(TestMatMPICRL_B_G23):
-        BSIZE = 5
-    class TestMatMPICRL_B_G45_B5(TestMatMPICRL_B_G45):
-        BSIZE = 5
-    class TestMatMPICRL_B_G89_B5(TestMatMPICRL_B_G89):
-        BSIZE = 5
-
-    # -----
-
 
 
 if __name__ == '__main__':
